<style>
	body {font-family:Arial;}
	table th, table td {border: 1px solid #ff960f;padding:10px;}
	table thead {background:#f1d7ce;}
	table tbody tr:nth-child(2n) {background:#f1d7ce4d;}
</style>
<?php
/* Funds Capture */
require 'vendor/autoload.php';
require 'config.php';

//Update Execution Time Limit
set_time_limit(600); 

function fetchAuthorizedOrders($shopify, $storeURL, $orderCursor = null, $lastId = null) {
/* This query fetches paginates through all authorized and capturable orders.
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, currentTotalPriceSet.shopMoney.amount
*/
	
//If Cursor, Add in pagination
if (!is_null($orderCursor) && !empty($orderCursor)) {
	$orderCursorText = ', after: "'.$orderCursor.'"';
} else {
	$orderCursorText = '';
}
		
$gQLRequest =
<<<Query
query {
  orders(first: 20, query: "(fulfillment_status:shipped) AND (financial_status:authorized)" $orderCursorText) {
	pageInfo {
		hasNextPage
		hasPreviousPage
	}
	edges {
	  cursor
	  node {
		id
		capturable
		transactions {
		  id
		}
		currentTotalPriceSet {
		  shopMoney {
			amount
		  }
		}
	  }
	}
  }
}
Query;
	$data = $shopify->GraphQL->post($gQLRequest);

	if ( count($data["data"]["orders"]["edges"]) > 0) {

		//Checks for paginatinated results
		$hasPreviousPage = $data["data"]["orders"]["pageInfo"]["hasPreviousPage"];
		$hasNextPage 	 = $data["data"]["orders"]["pageInfo"]["hasNextPage"];
		$orderCursor 	 = $data["data"]["orders"]["edges"][0]["cursor"];
		
		if ($hasPreviousPage == false || empty($hasPreviousPage)) {
			echo 
			"<table>
			<thead>
				<tr>
					<th>Order ID</th>
					<th>Transaction ID</th>
					<th>Capturable Total</th>
				</tr>
			</thead>
			<tbody>";
		}
		
		foreach($data["data"]["orders"]["edges"] as $order) {
			/*
			* 	For each order, capture payment
			*/
			// If order is not capturable, skip
			if ($order["node"]["capturable"] == false) {
				continue;
			}
			
			$transaction = end($order["node"]["transactions"]);
			$orderId  = $order["node"]["id"];
			$orderArr = explode("/", $orderId);
			$orderNum = end($orderArr);
			$updatedTotal = floatval($order["node"]["currentTotalPriceSet"]["shopMoney"]["amount"]);
			
			if ($lastId !== $orderId) {
				echo 
				"<tr>
					<td><a href=\"https://". $storeURL ."/admin/orders/".$orderNum."\">".$orderNum."</a></td>
					<td>".$transaction["id"]."</td>
					<td>$".number_format($updatedTotal, 2)."</td>
				</tr>";

				// Capture Payment
				captureFunds($shopify, $orderId, $transaction["id"], $updatedTotal);
			}
		}
		
		if ($hasNextPage == false || empty($hasNextPage)) {
			echo "</tbody></table>";
			echo "<h4>&#9989; All Eligible Orders Payments Have been Captured!</h4><a href=\"./fundsCapture.php\">Run Again?</a>";
		} else {
			//Process Next Set Of Orders
			fetchAuthorizedOrders($shopify, $storeURL, $orderCursor, $orderId);
		}
	}
	else {
		echo "<br/><br/><em>&#x274C; There are no Eligible Orders To Capture.</em><br/><a href=\"./fundsCapture.php\">Check Again?</a>";
	}
}
	
	
function captureFunds($shopify, $orderId, $transactionId, $orderTotal) {
/* This query fetches paginates through all authorized and capturable orders.
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
$input = '{ id:"'.$orderId.'", parentTransactionId:"'.$transactionId.'", amount:"'.$orderTotal.'" }';

$gQLRequest =
<<<Query
mutation {
  orderCapture(input: $input) {
    transaction {
      id
    }
    userErrors {
      field
      message
    }
  }
}
Query;

$data = $shopify->GraphQL->post($gQLRequest);	
//print("<pre>commitOrderUpdate ".print_r($data,true)."</pre>");
}
	
echo "<h2>NatureMade&reg; | Funds Capture</h2> <em>Store: ".$config["ShopUrl"]."</em>";
// Fetch Orders & Capture Authorized Payments	
fetchAuthorizedOrders($shopify, $config["ShopUrl"]);


?>