<style>
	body {font-family:Arial;}
	table th, table td {border: 1px solid #2e56a742;padding:10px;}
	table thead {background:#496a9c4f;}
	table tbody tr:nth-child(2n+1) {background:#ced6f14f;}
	strong {color:red;}
</style>
<?php
/* Funds Capture */
require 'vendor/autoload.php';
require 'config.php';

//Update Execution Time Limit
set_time_limit(600); 

function updatePartialLineItems($shopify, $storeURL, $orderCursor = null, $lastId = null) {
/* This query fetches paginates through all authorized and capturable orders.
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
	
//If Order Cursor, Add in pagination
$orderCursorText = (!is_null($orderCursor) && !empty($orderCursor)) ? ', after: "'. $orderCursor .'"' : '';

$gQLRequest =
<<<Query
query {
orders(first:20, query:"(fulfillment_status:partial) AND (financial_status:authorized)" $orderCursorText) {
pageInfo {
	  hasNextPage
	  hasPreviousPage
}
edges {
	cursor
	node {
	id
	displayFulfillmentStatus	
	displayFinancialStatus
	createdAt
	  lineItems (first:20) {
		pageInfo {
		  hasNextPage
		  hasPreviousPage
		}
		edges {
		  cursor
		  node {
			id
			fulfillmentStatus
			unfulfilledQuantity
			fulfillableQuantity
			quantity
			variant {
			  id
			  displayName
			}
		  }
		}
	  }
	}
}
}
}
Query;
	$data = $shopify->GraphQL->post($gQLRequest);
	
	if ((count($data["data"]["orders"]["edges"]) > 0)) {

		//Checks for paginatinated results
		$hasPreviousPage 	 = $data["data"]["orders"]["pageInfo"]["hasPreviousPage"];
		$hasNextPage 	 = $data["data"]["orders"]["pageInfo"]["hasNextPage"];
		$orderCursor 		 = $data["data"]["orders"]["edges"][0]["cursor"];
		
		if ($hasPreviousPage == false || empty($hasPreviousPage)) {
			echo 
			"<table>
			<thead>
				<tr>
					<th>Order ID</th>
					<th>Product - Variant</th>
					<th>Variant ID</th>
					<th>Line Item ID</th>
					<th>Total QTY Ordered</th>
					<th>Unfulfilled QTY</th>
					<th>New QTY</th>
				</tr>
			</thead>
			<tbody>";
		}
		
		foreach($data["data"]["orders"]["edges"] as $order) {
			
			// Traverse through each order
			// Used to create order url and as a cursor check
			$orderId = $order["node"]["id"];
			$orderArr = explode("/", $orderId);
			$orderNum = end($orderArr);
			
			// Fetch Calculated Order IDs & LineItems
			$calculatedOrder = fetchCalculated($shopify, $orderId);
			
			if ($lastId !== $orderId) {
				$lineItemHasNextPage 	 = $order["node"]["lineItems"]["pageInfo"]["hasNextPage"];
				
				foreach($order["node"]["lineItems"]["edges"] as $lineItem) {
					// Traverse through every line item 
					// Fulfillable Quantity = 0 means that the item may be removed.
					if ( intval($lineItem["node"]["fulfillableQuantity"]) > 0 && intval($lineItem["node"]["unfulfilledQuantity"]) > 0) {
						
						//Get updated order quantity (based on actual fulfilled amount)
							$newQTY = intval($lineItem["node"]["quantity"]) - intval($lineItem["node"]["unfulfilledQuantity"]);
						
						echo 
						"<tr>
							<td><a href=\"https://". $storeURL ."/admin/orders/".$orderNum."\">".$orderNum."</a></td>
							<th>".$lineItem["node"]["variant"]["displayName"]."</th>
							<th>".$lineItem["node"]["variant"]["id"]."</th>
							<th>".$lineItem["node"]["id"]."</th>
							<th>".$lineItem["node"]["quantity"]."</th>
							<th>".$lineItem["node"]["unfulfilledQuantity"]."</th>
							<th><strong>".$newQTY."</strong></th>
						</tr>";
						
						
						if ($lineItem["node"]["fulfillmentStatus"] == "partial") {
							// Sets the Fulfillable Quantity to Zero for parital line items.
							$newQTY = 0;
						}
						// Update Line Items
						updateLineItemQuantity($shopify, $calculatedOrder["calculatedOrderId"], $calculatedOrder["calculatedLineItem"][$lineItem["node"]["variant"]["id"]], $newQTY);
					
					}
					
					$lineItemCursor = $lineItem["cursor"];
				}
				
				//Process Next LineItem Batch from current order
				if (isset($lineItemCursor) && !empty($lineItemHasNextPage) && $lineItemHasNextPage !== false) {
					paginateLineItems($shopify, $storeURL, $orderId, $lineItemCursor);
				}
			}
			// Commit All CalculatedLineItems Changes to the Same CalculatedOrder
			commitOrderChanges($shopify, $calculatedOrder["calculatedOrderId"]);
		}
		// Checks if there are orders to paginate through, or shows the end of the table.
		if ($hasNextPage == false || empty($hasNextPage)) {
			echo "</tbody></table>";
			echo "<h4>&#9989; All Partially Fulfilled Line Items Have Been Updated!</h4><a href=\"./updatePartialLineItems.php\">Run Again?</a>";

		} else {
			//Process Next Batch Of Orders
			updatePartialLineItems($shopify, $storeURL, $orderCursor, $orderId);
		}
	}
	else {
		echo "<br/><br/><em>&#x274C; There are no Authorized Partially Fulfilled Orders.</em><br/><a href=\"./updatePartialLineItems.php\">Check Again?</a>";
	}
}


function paginateLineItems($shopify, $storeURL, $orderId = null, $lineItemCursor = null, $lastLineItemId = null) {
/* This query fetches paginates through all remaining line items from a Single Order
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
	
//If LineItem Cursor, Add in pagination
$lineItemCursorText = (!is_null($lineItemCursor) && !empty($lineItemCursor)) ? ', after: "'.$lineItemCursor .'"' : '';


$gQLRequest =
<<<Query
query {
	order(id: "$orderId") {
	id
	displayFulfillmentStatus	
	displayFinancialStatus
	createdAt
	lineItems (first:20 $lineItemCursorText) {
		pageInfo {
		  hasNextPage
		  hasPreviousPage
		}
		edges {
		  cursor
		  node {
			id
			fulfillmentStatus
			unfulfilledQuantity
			fulfillableQuantity
			quantity
			variant {
			  id
			  displayName
			}
		  }
		}
	}
	}
}
Query;
	$data = $shopify->GraphQL->post($gQLRequest);

	// Used to create order url
	$orderId  = $data["data"]["order"]["id"];
	$orderArr = explode("/", $orderId);
	$orderNum = end($orderArr);
		
	
			// Fetch Calculated Order IDs & LineItems
			$calculatedOrder = fetchCalculated($shopify, $orderId);
			
	//Checks for paginatinated LineItem results
	$lineItemHasNextPage = $data["data"]["order"]["lineItems"]["pageInfo"]["hasNextPage"];
	
	foreach($data["data"]["order"]["lineItems"]["edges"] as $lineItem) {
		// Traverse through every lineItem
		if ($lastLineItemId !== $lineItem["node"]["id"]) {
			// Fulfillable Quantity = 0 means that the item may have been removed already.
			if (intval($lineItem["node"]["fulfillableQuantity"]) > 0 && intval($lineItem["node"]["unfulfilledQuantity"]) > 0) {
									
				//Get updated order quantity (based on actual fulfilled amount)
				$newQTY = intval($lineItem["node"]["quantity"]) - intval($lineItem["node"]["unfulfilledQuantity"]);
				
				echo 
				"<tr>
					<td><a href=\"https://". $storeURL ."/admin/orders/".$orderNum."\">".$orderNum."</a></td>
					<th>".$lineItem["node"]["variant"]["displayName"]."</th>
					<th>".$lineItem["node"]["variant"]["id"]."</th>
					<th>".$lineItem["node"]["id"]."</th>
					<th>".$lineItem["node"]["quantity"]."</th>
					<th>".$lineItem["node"]["unfulfilledQuantity"]."</th>
					<th><strong>".$newQTY."</strong></th>
				</tr>";
				
				if ($lineItem["node"]["fulfillmentStatus"] == "partial") {
					// Sets the Fulfillable Quantity to Zero for parital line items.
					$newQTY = 0;
				}

				// Update Line Items Only if Unfulfilled > 0
				updateLineItemQuantity($shopify, $calculatedOrder["calculatedOrderId"], $calculatedOrder["calculatedLineItem"][$lineItem["node"]["variant"]["id"]], $newQTY);
			}
		}
		$lineItemCursor = $lineItem["cursor"];
		$lastLineItemId = $lineItem["node"]["id"];
	}
	
	commitOrderChanges($shopify, $calculatedOrder["calculatedOrderId"]);
			
	// Get lineItem Pagination
	// Process Next LineItem Batch from current order
	if (!empty($lineItemHasNextPage) && $lineItemHasNextPage !== false) {
		paginateLineItems($shopify, $storeURL, $orderId, $lineItemCursor, $lastLineItemId);
	}
}

function fetchCalculated($shopify, $orderId) {
	/* Fetch Caclulated Orders and LineItems
	*  https://shopify.dev/docs/admin-api/graphql/reference/orders/ordereditbegin
	*/
	
$gQLRequest =
<<<Query
mutation {
  orderEditBegin(id: "$orderId") {
    calculatedOrder {
      id
      lineItems(first: 100) {
        edges {
          node {
            id
           variant {
            id
          }
          }
        }
      }
    }
    userErrors {
      field
      message
    }
  }
}
Query;

$data = $shopify->GraphQL->post($gQLRequest);	

	/*
	* Format data array return
	* Example Returned Array Format
	* Array (
	*     [calculatedOrderId] => gid://shopify/CalculatedOrder/4509597793
	*     [calculatedLineItem] => Array
	*         (
	*             [gid://shopify/ProductVariant/32713096101985] => gid://shopify/CalculatedLineItem/6131860340833
	*             [gid://shopify/ProductVariant/32713100296289] => gid://shopify/CalculatedLineItem/6131860373601
	*         )
	* )
	*/

	$calculatedArr["calculatedOrderId"] = $data["data"]["orderEditBegin"]["calculatedOrder"]["id"];
	foreach ($data["data"]["orderEditBegin"]["calculatedOrder"]["lineItems"]["edges"] as $calculatedLineItem) {
		$calculatedArr["calculatedLineItem"][$calculatedLineItem["node"]["variant"]["id"]] = $calculatedLineItem["node"]["id"];
	}	
										
	return $calculatedArr;
}


function updateLineItemQuantity($shopify, $calculatedOrderId, $calculatedLineItemId, $lineItemQty) {
/* Update Line Item Quantity in an order
*  https://shopify.dev/docs/admin-api/graphql/reference/orders/ordereditsetquantity
*  https://shopify.dev/docs/admin-api/graphql/reference/orders/ordereditcommit
*/

// Update Order QTY on Calculated LineItem	
$gQLRequest =
<<<Query
mutation {
  orderEditSetQuantity(id: "$calculatedOrderId", lineItemId: "$calculatedLineItemId", quantity: $lineItemQty) {
    calculatedLineItem {
      id
    }
    calculatedOrder {
      id
    }
    userErrors {
      field
      message
    }
  }
}
Query;

$data = $shopify->GraphQL->post($gQLRequest);	

}

function commitOrderChanges($shopify, $calculatedOrderId) {
// Commit Order Change

$gQLRequest =
<<<Query
mutation {
  orderEditCommit(id:"$calculatedOrderId", notifyCustomer:false) {
    order {
      id
    }
    userErrors {
      field
      message
    }
  }
}
Query;
$data = $shopify->GraphQL->post($gQLRequest);	

}

echo "<h2>NatureMade&reg; | Update Partially Fulfilled Orders</h2> <em>Store: ".$config["ShopUrl"]."</em>";

// Fetch Orders & Capture Authorized Payments	
updatePartialLineItems($shopify, $config["ShopUrl"]);

?>
